/*
 * store_aufs.h
 *
 * Internal declarations for the aufs routines
 */

#ifndef __STORE_ASYNCUFS_H__
#define __STORE_ASYNCUFS_H__

#ifdef ASYNC_IO_THREADS
#define NUMTHREADS ASYNC_IO_THREADS
#else
#define NUMTHREADS 16
#endif

#define MAGIC1 (NUMTHREADS/Config.cacheSwap.n_configured/2)

struct _aio_result_t {
    int aio_return;
    int aio_errno;
};

typedef struct _aio_result_t aio_result_t;

typedef void AIOCB(int fd, void *, int aio_return, int aio_errno);

int squid_aio_cancel(aio_result_t *);
int squid_aio_open(const char *, int, mode_t, aio_result_t *);
int squid_aio_read(int, char *, int, off_t, int, aio_result_t *);
int squid_aio_write(int, char *, int, off_t, int, aio_result_t *);
int squid_aio_close(int, aio_result_t *);
int squid_aio_stat(const char *, struct stat *, aio_result_t *);
int squid_aio_unlink(const char *, aio_result_t *);
int squid_aio_truncate(const char *, off_t length, aio_result_t *);
int squid_aio_opendir(const char *, aio_result_t *);
aio_result_t *squid_aio_poll_done(void);
int squid_aio_operations_pending(void);
int squid_aio_overloaded(void);
int squid_aio_sync(void);
int squid_aio_get_queue_len(void);

void squidAioInit(void);
void squidAioDone(void);
void squidAioCancel(int);
void squidAioOpen(const char *, int, mode_t, AIOCB *, void *);
void squidAioClose(int);
void squidAioWrite(int, int offset, char *, int size, AIOCB *, void *, FREE *);
void squidAioRead(int, int offset, char *, int size, AIOCB *, void *);
void squidAioStat(char *, struct stat *, AIOCB *, void *);
void squidAioUnlink(const char *, AIOCB *, void *);
void squidAioTruncate(const char *, off_t length, AIOCB *, void *);
int squidAioCheckCallbacks(SwapDir *);
void squidAioSync(SwapDir *);
int squidAioQueueSize(void);

struct _squid_aioinfo_t {
    int swaplog_fd;
    int l1;
    int l2;
    fileMap *map;
    int suggest;
};

struct _squid_aiostate_t {
    int fd;
    struct {
	unsigned int close_request:1;
	unsigned int reading:1;
	unsigned int writing:1;
	unsigned int opening:1;
    } flags;
    const char *read_buf;
    link_list *pending_writes;
    link_list *pending_reads;
};

typedef struct _squid_aioinfo_t squid_aioinfo_t;
typedef struct _squid_aiostate_t squid_aiostate_t;

/* The aio_state memory pool */
extern MemPool *squid_aio_state_pool;

extern void storeAufsDirMapBitReset(SwapDir *, sfileno);
extern int storeAufsDirMapBitAllocate(SwapDir *);

extern char *storeAufsDirFullPath(SwapDir * SD, sfileno filn, char *fullpath);
extern void storeAufsDirUnlinkFile(SwapDir *, sfileno);
extern void storeAufsDirReplAdd(SwapDir * SD, StoreEntry *);
extern void storeAufsDirReplRemove(StoreEntry *);

/*
 * Store IO stuff
 */
extern STOBJCREATE storeAufsCreate;
extern STOBJOPEN storeAufsOpen;
extern STOBJCLOSE storeAufsClose;
extern STOBJREAD storeAufsRead;
extern STOBJWRITE storeAufsWrite;
extern STOBJUNLINK storeAufsUnlink;

#endif
